"""
This type stub file was generated by pyright.
"""

from ._lattice_ecp5 import LatticeECP5Platform
from ._lattice_ice40 import LatticeICE40Platform

__all__ = ["LatticeECP5Platform", "LatticeICE40Platform"]
